#include <cuda_fp16.h>

{{corr_enum}}

extern "C" {
	__global__ void correct(const {{input_data_dtype}}* data, // shape: memory cell, 1, y, x (I think)
	                        const unsigned short* cell_table,
	                        const unsigned char corr_flags,
	                        const float* offset_map, // shape: cell, y, x, gain
	                        const float* gain_amp_map,
	                        const float* rel_gain_slopes_map,
	                        const float* flatfield_map,
	                        const unsigned int* bad_pixel_map,
	                        const float bad_pixel_mask_value,
	                        {{output_data_dtype}}* output) {
		const size_t X = {{pixels_x}};
		const size_t Y = {{pixels_y}};
		const size_t memory_cells = {{data_memory_cells}};
		const size_t map_memory_cells = {{constant_memory_cells}};

		const size_t memory_cell = blockIdx.x * blockDim.x + threadIdx.x;
		const size_t y = blockIdx.y * blockDim.y + threadIdx.y;
		const size_t x = blockIdx.z * blockDim.z + threadIdx.z;

		if (memory_cell >= memory_cells || y >= Y || x >= X) {
			return;
		}

		const size_t data_stride_x = 1;
		const size_t data_stride_y = X * data_stride_x;
		const size_t data_stride_cell = Y * data_stride_y;
		const size_t data_index = memory_cell * data_stride_cell + y * data_stride_y + x * data_stride_x;
		const unsigned short raw_data_value = data[data_index];
		const unsigned char gain = raw_data_value >> 12;
		float corrected = (float)(raw_data_value & 0xfff);

		const size_t gm_map_stride_gain = 1;
		const size_t gm_map_stride_x = 3 * gm_map_stride_gain;
		const size_t gm_map_stride_y = X * gm_map_stride_x;
		const size_t gm_map_stride_cell = Y * gm_map_stride_y;

		const size_t map_cell = cell_table[memory_cell];
		if (map_cell < map_memory_cells) {
			const size_t gm_map_index = gain * gm_map_stride_gain +
				map_cell * gm_map_stride_cell +
				y * gm_map_stride_y +
				x * gm_map_stride_x;

			if ((corr_flags & BPMASK) && bad_pixel_map[gm_map_index]) {
				corrected = bad_pixel_mask_value;
			} else {
				// TODO: save GPU memory by combining maps
				if (corr_flags & OFFSET) {
					corrected -= offset_map[gm_map_index];
				}
				if (corr_flags & GAIN_AMP) {
					corrected *= gain_amp_map[gm_map_index];
				}
				if (corr_flags & REL_GAIN) {
					corrected *= rel_gain_slopes_map[gm_map_index];
				}
				if (corr_flags & FF_CORR) {
					corrected *= flatfield_map[gm_map_index];
				}
			}
		}
		{% if output_data_dtype == "half" %}
		output[data_index] = __float2half(corrected);
		{% else %}
		output[data_index] = ({{output_data_dtype}})corrected;
		{% endif %}
	}
}
